# Generated by Django 3.0.3 on 2020-03-17 17:19

from django.db import migrations, models
import json
import os
from pathlib import Path

with open(
    os.path.join(Path(__file__).parent.absolute(), "usersettings.json"), "r"
) as fh:
    data = json.load(fh)


class Migration(migrations.Migration):
    dependencies = [("api", "0002_jobtemplate")]
    operations = [
        migrations.RemoveField(model_name="usersettings", name="max_notifs"),
        migrations.RemoveField(model_name="usersettings", name="notifs_created"),
        migrations.RemoveField(model_name="usersettings", name="notifs_event"),
        migrations.RemoveField(model_name="usersettings", name="notifs_published"),
        migrations.RemoveField(model_name="usersettings", name="notifs_returned"),
        migrations.AddField(
            model_name="usersettings",
            name="settings",
            field=models.JSONField(default=data),
        ),
    ]
